import {Component, EventEmitter, Input, OnInit, Output} from '@angular/core';
import {State} from "../../project/game-model/gamesystems/states/State";
import {Gamesystem} from "../../project/game-model/gamesystems/Gamesystem";
import { Transition } from '../../project/game-model/gamesystems/transitions/Transition';
import {ScriptAccount} from "../../project/game-model/scriptAccounts/ScriptAccount";
import {SimpleGamesystem} from "../../project/game-model/gamesystems/SimpleGamesystem";
import {ProductGamesystem} from "../../project/game-model/gamesystems/ProductGamesystem";
import {TemplateElement} from "../../project/game-model/templates/TemplateElement";
import {SimpleTemplateGamesystem} from "../../project/game-model/templates/simpleGamesystem/SimpleTemplateGamesystem";
import {ProductTemplateSystem} from "../../project/game-model/templates/productGamesystem/ProductTemplateSystem";

@Component({
  selector: 'app-gamesystem-editor',
  templateUrl: './gamesystem-editor.component.html',
  styleUrl: './gamesystem-editor.component.scss'
})
export class GamesystemEditorComponent implements OnInit{

  @Input() gamesystem: Gamesystem<State<any>, Transition<any>> | undefined
  @Input() scriptAccounts: ScriptAccount[] = [];
  @Input() templateElement: TemplateElement | undefined
  @Output('onOpenGamesystemEditor') openGamesystemEmitter = new EventEmitter<SimpleGamesystem>();

  ngOnInit() {
    console.log("GamesystemEditor: ", this.scriptAccounts.length)
  }

  isSimpleGamesystem() {
    return this.gamesystem instanceof SimpleGamesystem;
  }

  convertGamesystemToSimpleGamesystem() {
    if(this.gamesystem instanceof SimpleGamesystem) {
      return this.gamesystem as SimpleGamesystem;
    }
  }

  convertGamesystemToProductGamesystem() {
    if(this.gamesystem instanceof ProductGamesystem) {
      return this.gamesystem as ProductGamesystem;
    }
  }

  onOpenGamesystemEditor(gamesystem: SimpleGamesystem) {
    this.openGamesystemEmitter.emit(gamesystem);
  }

  isGamesystemTemplate() {
    return this.gamesystem instanceof SimpleTemplateGamesystem || this.gamesystem instanceof ProductTemplateSystem;
  }

  convertGamesystemToTemplate(gamesystem: Gamesystem<State<any>, Transition<any>> | undefined) {
    if(gamesystem instanceof SimpleTemplateGamesystem) {
      return gamesystem as SimpleTemplateGamesystem;
    } else if(gamesystem instanceof ProductTemplateSystem) {
      return gamesystem as ProductTemplateSystem
    } else {
      return undefined
    }

  }
}
