import {Component, Input} from '@angular/core';
import {Item} from "../../../project/game-model/inventory/Item";
import {ItemGroup} from "../../../project/game-model/inventory/ItemGroup";
import {GameModel} from "../../../project/game-model/GameModel";
import {MatDialog} from "@angular/material/dialog";
import {ItemgroupInheritorComponent} from "./itemgroup-inheritor/itemgroup-inheritor.component";
import {ItemgroupUtilities} from "../../../project/game-model/utils/ItemgroupUtilities";

@Component({
  selector: 'app-item-editor',
  templateUrl: './item-editor.component.html',
  styleUrl: './item-editor.component.scss'
})
export class ItemEditorComponent {

  @Input() item: Item | undefined
  @Input() gameModel: GameModel | undefined;


  constructor(private dialog: MatDialog) {

  }

  onAddNewInheritedItemgroup() {
    const itemgroups = this.item!.manuallyInheritedGroups.concat(this.item!.hierarchyInheritedGroups);
    const dialogRef = this.dialog.open(ItemgroupInheritorComponent, {
      data: this.gameModel!.itemgroupsAsList.filter(group => !itemgroups.includes(group)),
      width: "400px"
    })

    dialogRef.afterClosed().subscribe(res => {
      if(res != undefined) {
        this.item!.addInheritedGroup(res);
      }
    })
  }

  deleteInheritedItemgroup(itemgroup: ItemGroup) {
    this.item!.deleteInheritedGroup(itemgroup);
  }

  get automaticallyInheritedItemgroups() {
    return ItemgroupUtilities.findItemgroupPathToItem(this.item!.componentName, this.gameModel!.itemgroups);
  }

  get manuallyAndHierarchyItemgroups() {
    return this.item!.hierarchyInheritedGroups.concat(this.item!.manuallyInheritedGroups);
  }
}
