import {IsolatedProductStateGenerator} from "./IsolatedProductStateGenerator";
import {Transition} from "../transitions/Transition";
import {SimpleTemplateTransition} from "../../templates/simpleGamesystem/SimpleTemplateTransition";
import {State} from "../states/State";
import {ScriptAccountCondition} from "../conditions/ScriptAccountCondition";
import {SimpleTemplateState} from "../../templates/simpleGamesystem/SimpleTemplateState";
import {Character} from "../../characters/Character";
import {CharacterRelation} from "../../characters/CharacterRelation";
import {TemplateElement} from "../../templates/TemplateElement";
import {ProductTemplateSystem} from "../../templates/productGamesystem/ProductTemplateSystem";
import {ProductGeneratorResult} from "./ProductGeneratorResult";
import {state, transition} from "@angular/animations";

export class IsolatedTemplateStateGenerator extends IsolatedProductStateGenerator {
  templateElement: TemplateElement

  constructor(productGamesystem: ProductTemplateSystem, templateElement: TemplateElement) {
    super(productGamesystem);
    this.templateElement = templateElement;
  }


  protected assignGeneratedStatesAndTransitions(generationResult: ProductGeneratorResult) {
    const templateSystem = this.productGamesystem as ProductTemplateSystem;

    const nonIsolatedStates = templateSystem.stateMap.get(this.templateElement)!
    const states = nonIsolatedStates.concat(generationResult.states);
    templateSystem.stateMap.set(this.templateElement, states)
  }

  protected getTransitionConditions(transition: Transition<any>, leftSystem: boolean) {
    const templateElement = this.determineTemplateElement(leftSystem)!;
    if(transition instanceof SimpleTemplateTransition && transition.conditionMap.has(templateElement)) {
      return transition.conditionMap.get(templateElement)!
    }
    return transition.scriptAccountConditions;
  }

  protected getTransitionActions(transition: Transition<any>, leftSystem: boolean) {
    const templateElement = this.determineTemplateElement(leftSystem)!;

    if(transition instanceof SimpleTemplateTransition && transition.actionMap.has(templateElement)) {
      return transition.actionMap.get(templateElement)!
    } else {
      return transition.scriptAccountActions;
    }
  }


  protected getStateConditions(state: State<any>, leftSystem: boolean): ScriptAccountCondition[] {
    const templateElement = this.determineTemplateElement(leftSystem)!
    if(state instanceof  SimpleTemplateState && state.conditionMap.has(templateElement)) {
      return state.conditionMap.get(templateElement)!
    } else {
      return state.conditions
    }
  }

  private determineTemplateElement(leftSystem: boolean) {
    if(this.templateElement instanceof Character) {
      return this.templateElement;
    } else if(this.templateElement instanceof CharacterRelation) {
      return leftSystem ? this.templateElement.firstCharacter : this.templateElement.secondCharacter;
    }
  }
}
