import {TemplateProductSystemGenerator} from "./TemplateProductSystemGenerator";
import {ProductGenerationData} from "./ProductGenerationData";
import {ProductGeneratorResult} from "./ProductGeneratorResult";
import {ProductState} from "../states/ProductState";
import {ProductTransition} from "../transitions/ProductTransition";
import {ProductTemplateSystem} from "../../templates/productGamesystem/ProductTemplateSystem";

export class SymmetricProductTemplateGenerator extends TemplateProductSystemGenerator {
  protected generateFromBinaryChildsystems(leftSystemData: ProductGenerationData, rightSystemData: ProductGenerationData): ProductGeneratorResult {
    const generatedProductStates: ProductState[] = []
    const generatedProductTransitions: ProductTransition[] = []

    leftSystemData.states.forEach(leftState => {
      rightSystemData.states.forEach(rightState => {
        if(leftState.equals(rightState)) {
          for(let i=0; i<leftState.outgoingTransitions.length; i++) {
            for(let j=0; j<rightState.outgoingTransitions.length; j++) {
              const startingState = this.generateBinaryProductState(leftState, rightState, generatedProductStates);
              if(startingState != undefined && leftState.outgoingTransitions[i].endingState.equals(rightState.outgoingTransitions[j].endingState)) {
                const endingState_left_right = this.generateBinaryProductState(leftState.outgoingTransitions[i].endingState, rightState.outgoingTransitions[j].endingState, generatedProductStates);
                if(endingState_left_right != undefined) {
                  this.generateBinaryProductTransitionMulti(startingState, endingState_left_right, leftState.outgoingTransitions[i], rightState.outgoingTransitions[j], generatedProductTransitions);
                }
              }
            }
          }
        }
      })
    })

    return new ProductGeneratorResult(generatedProductStates, generatedProductTransitions);
  }

}
