import {ItemGroup} from "../../game-model/inventory/ItemGroup";
import {StoreComponent} from "../../../../../app/storage/StoreComponent";
import {ItemGroupCharacteristic} from "../../game-model/inventory/ItemgroupCharacteristic";
import {AbstractItemGroup} from "../../game-model/inventory/AbstractItemGroup";
import {ModelComponentType} from "../../game-model/ModelComponentType";
import {ConcreteItemGroup} from "../../game-model/inventory/ConcreteItemGroup";

export class ItemgroupParser {

  private parsedItemgroups: ItemGroup[] = []
  private topParsedItemgroups: ItemGroup[] = []

  parseItemgroups(storeComponents: StoreComponent[]) {
    const topologicalSort = this.topologicalSort(storeComponents);
    topologicalSort.forEach(storeComponent => this.parseSingleItemgroup(storeComponent));
    return this.parsedItemgroups;
  }

  private topologicalSort(storeComponents: StoreComponent[]) {
    const sortedObjects: StoreComponent[] = []
    const visited: {[key: string]: boolean} = {};

    const visit = (object: StoreComponent) => {
      const jsonData = JSON.parse(object.jsonString)
      if(visited[jsonData.componentName]) return;
      visited[jsonData.componentName] = true;

      if(jsonData.parent) {
        const parentObject = storeComponents.find(obj => JSON.parse(obj.jsonString).componentName === jsonData.parent);
        if(parentObject) {
          visit(parentObject);
        }
      }
      sortedObjects.push(object);
    }

    storeComponents.forEach(object => visit(object));

    return sortedObjects;
  }

  parseSingleItemgroup(storeComponent: StoreComponent) {
    const data = JSON.parse(storeComponent.jsonString);

    const componentName = data.componentName;
    const componentDescription = data.componentDescription;
    const parentgroupName = data.parentgroup;

    let itemgroup;
    if(data.itemgroupType == 0) {
      itemgroup = new ConcreteItemGroup(componentName, componentDescription, ModelComponentType.ITEMGROUP);
    } else {
      itemgroup = new AbstractItemGroup(componentName, componentDescription, ModelComponentType.ITEMGROUP);
    }

    const parentgroup = this.parsedItemgroups.find(parent => parent.componentName === parentgroupName);

    if(parentgroup != undefined) {
      const abstractParentgroup = parentgroup as AbstractItemGroup;
      abstractParentgroup.addChildItemgroup(itemgroup)
    } else {
      this.parsedItemgroups.push(itemgroup);
      this.topParsedItemgroups.push(itemgroup);
    }

    this.parseItemgroupCharacteristics(data.itemGroupCharacteristics, itemgroup);
  }

  parseItemgroupCharacteristics(itemgroupCharacteristicsData: any, itemgroup: ItemGroup): ItemGroupCharacteristic[] {
    const itemgroupCharacteristics: ItemGroupCharacteristic[] = []
    for(let i=0; i<itemgroupCharacteristicsData.length; i++) {
      this.parseSingleItemgroupCharacteristic(itemgroupCharacteristicsData[i], itemgroup)
    }
    return itemgroupCharacteristics;
  }

  private parseSingleItemgroupCharacteristic(itemgroupCharacteristicData: any, itemgroup: ItemGroup) {
    const name = itemgroupCharacteristicData.characteristicName;
    const description = itemgroupCharacteristicData.characteristicDescription;

    return new ItemGroupCharacteristic(name, description, itemgroup);
  }

  getParsedTopItemgroups() {
    return this.topParsedItemgroups;
  }

}
